import 'dart:async';
import 'package:built_redux/built_redux.dart';
import 'package:wui_builder/wui_builder.dart';
import 'package:wui_builder/functional.dart';
import 'package:built_value/built_value.dart';

const builtReduxContextKey = 'builtReduxContextKey';

class ReduxProviderProps<V extends Built<V, B>, B extends Builder<V, B>,
    A extends ReduxActions> {
  Store<V, B, A> store;
  VNode child;
}

class ReduxProvider<V extends Built<V, B>, B extends Builder<V, B>,
        A extends ReduxActions>
    extends Component<ReduxProviderProps<V, B, A>, Null> {
  ReduxProvider(ReduxProviderProps<V, B, A> props) : super(props);

  @override
  Map<String, dynamic> getChildContext() => <String, dynamic>{
        builtReduxContextKey: props.store,
      };

  @override
  VNode render() => props.child;
}

typedef InnerP Connect<V extends Built<V, B>, B extends Builder<V, B>,
    A extends ReduxActions, OutterP, InnerP>(V state, OutterP ownProps);

class ReduxHOCProps<V extends Built<V, B>, B extends Builder<V, B>,
    A extends ReduxActions, OutterP, InnerP> {
  FunctionalComponent<InnerP> child;
  Connect<V, B, A, OutterP, InnerP> connect;
  OutterP ownProps;
}

class ReduxHOC<
    V extends Built<V, B>,
    B extends Builder<V, B>,
    A extends ReduxActions,
    OutterP,
    InnerP> extends Component<ReduxHOCProps<V, B, A, OutterP, InnerP>, InnerP> {
  StreamSubscription _sub;

  ReduxHOC(ReduxHOCProps<V, B, A, OutterP, InnerP> props) : super(props);

  @override
  InnerP getInitialState() => props.connect(
        (context[builtReduxContextKey] as Store<V, B, A>).state,
        props.ownProps,
      );

  @override
  void componentDidMount() {
    final store = context[builtReduxContextKey] as Store<V, B, A>;
    _sub = store.stream.listen((change) {
      final next = props.connect(store.state, props.ownProps);
      if (state != next) update((_, __) => next);
    });
  }

  @override
  void componentWillUnmount() {
    _sub.cancel();
  }

  @override
  VNode render() => props.child(state);
}
